require 'cgi'
require 'httparty'
require 'json'
require 'uri'
require 'yaml'

module Homophone
  module Service
    module MusicianRetriever
      def musician(name)
        data = get("/search?type=artist&q=#{name}")['artists']['items']
        return nil unless data.count > 0
        data = data[0]
        Musician.new(self, data)
      end
    end

    class Spotify
      include Homophone::Service::MusicianRetriever
      include HTTParty
      base_uri 'https://api.spotify.com/v1'

      def get(url)
        JSON.parse(self.class.get(url).body)
      end
    end

    class DummySpotify
      include Homophone::Service::MusicianRetriever

      def get(url)
        uri = URI(url)
        params = begin CGI::parse(uri.query) rescue nil end
        path_parts = uri.path.split('/').reject { |p| p == '' }
        path = cassette_path(path_parts, params)
        raise ArgumentError, "No cassette for #{url}" unless path
        path = File.join(source_path, "#{path}.yml")
        JSON.load(YAML.load(IO.read(path))['http_interactions'][0]['response']['body']['string'])
      end

      private

      def source_path
        File.join(File.dirname(__FILE__), '..', '..', 'features', 'fixtures', 'cassettes')
      end

      def cassette_path(components, params)
        case components.first
        when 'search'
          type = params['type'].first
          name = params['q'].first.gsub(/[+ ]/, '_')
          "search/#{type}/#{name}"
        when 'artists'
          "artists/#{components.last}/#{components[1]}"
        else
          nil
        end
      end
    end
  end

  class Musician
    attr_reader :spotify_id, :name, :genres

    def initialize(service, blob)
      @service = service
      @spotify_id = blob['id']
      @name = blob['name']
      @genres = blob['genres']
    end

    def related_artists
      data = @service.get("/artists/#{spotify_id}/related-artists")['artists']
      data.map { |e| Musician.new(@service, e) }
    end
  end
end
